#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "gsc_act.h"

#define PI 3.1415926535897932384626433

void err_exit( void)
{
   printf( "GSC11ACT needs,  as command-line arguments,  the name of an\n");
   printf( "input file of GSC 1.1 data,  in the original format;  and the\n");
   printf( "name of the output file of corrected data.\n");
   exit( 0);
}

void main( int argc, char **argv)
{
   FILE *ifile, *ofile, *plate_data_file = fopen( "gsc_act.dat", "rb");
   char buff[45], prev_buff1[45], prev_buff2[45];
   const int plate_cache_size = 7;
   long n_lines, line_no = 0L, dots_shown = 0;
   double rms1 = 0., rms2 = 0.;
   int n_overlapped = 0;
   PLATE_STRUCT *plate_array = (PLATE_STRUCT *)calloc( plate_cache_size,
                                                sizeof( PLATE_STRUCT));

   if( !plate_data_file)
      printf( "GSC_ACT.DAT not found!\n");
   if( argc < 3)
      err_exit( );
   ifile = fopen( argv[1], "rb");

   if( !ifile)
      {
      printf( "'%s' not opened\n", argv[1]);
      err_exit( );
      }

   ofile = fopen( argv[2], "wb");
   if( !ofile)
      {
      printf( "'%s' not opened\n", argv[2]);
      err_exit( );
      }

   if( !plate_data_file)
      exit( 0);

   fseek( ifile, 0L, SEEK_END);
   n_lines = ftell( ifile) / 45L;
   fseek( ifile, 0L, SEEK_SET);
   printf( "%ld lines in %s\n", n_lines, argv[1]);
   *prev_buff1 = *prev_buff2 = '\0';
   while( fread( buff, 45, 1, ifile))
      {

      line_no++;
      if( line_no >= 192 && buff[4] != ' ')
         {
         char plate_name[4];

         memcpy( plate_name, buff + 41, 3);
         plate_name[3] = '\0';
         if( !gsc_act_cached_load( plate_data_file, plate_name,
                                         plate_array, plate_cache_size))
            {
            double ra = atof( buff + 5) * PI / 180.;
            double dec = atof( buff + 14) * PI / 180.;
            double ra_out, dec_out;
            char temp_char = buff[23];
            char original_buff[45];

            memcpy( original_buff, buff, 45);
            apply_transformation( ra, dec, &ra_out, &dec_out, plate_array);
            if( ra_out >= PI * 2.)
               ra_out -= PI * 2.;
            if( ra_out < 0.)
               ra_out += PI * 2.;
            sprintf( buff + 5, "%9.5lf%9.5lf", ra_out * 180. / PI,
                                          dec_out * 180. / PI);
            buff[23] = temp_char;
            if( !memcmp( prev_buff2, buff, 5))
               {
               double d_ra1  = atof( prev_buff1 + 5) - ra * 180. / PI;
               double d_dec1 = atof( prev_buff1 + 14) - dec * 180. / PI;
               double d_ra2  = atof( prev_buff2 + 5) - ra_out * 180. / PI;
               double d_dec2 = atof( prev_buff2 + 14) - dec_out * 180. / PI;

               d_ra1 *= cos( dec);
               d_ra2 *= cos( dec);
               rms1 += d_ra1 * d_ra1 + d_dec1 * d_dec1;
               rms2 += d_ra2 * d_ra2 + d_dec2 * d_dec2;
               n_overlapped++;
               }
            memcpy( prev_buff1, original_buff, 45);
            memcpy( prev_buff2, buff, 45);
            }
         }
      fwrite( buff, 45, 1, ofile);
      while( dots_shown < 79 * line_no / n_lines)
         {
         printf( ".");
         dots_shown++;
         }
      }

   fclose( ifile);
   fclose( ofile);
   fclose( plate_data_file);
   free( plate_array);
   printf( "\n");

   if( n_overlapped)
      printf( "%d overlapping stars found; rms went from %.3lf to %.3lf\n",
               n_overlapped,
               3600. * sqrt( rms1 / (double)n_overlapped),
               3600. * sqrt( rms2 / (double)n_overlapped));
}
